from django.contrib.messages.views import SuccessMessageMixin
from django.http import JsonResponse
from django.urls import path, include
from .views import registration,user_login,user_logout, verify_otp, resend_otp, profile
from django.contrib.auth import views as auth_views


class CustomPasswordChangeView(SuccessMessageMixin, auth_views.PasswordChangeView):
 template_name = "main.html"
 success_message = "Your password has been successfully changed!"
 success_url = "/user/change-password/done"

 def form_valid(self, form):
  """Handles successful password change via AJAX."""
  response = super().form_valid(form)
  return JsonResponse({"success": True, "message": self.success_message})


 def form_invalid(self, form):
  """Handles errors when password change fails via AJAX."""
  return JsonResponse({"success": False, "errors": form.errors}, status=400)

urlpatterns = [
 path('registration', registration,name="register"),
 path('login', user_login,name="login"),
 path('logout', user_logout,name="logout"),
 path('mail/verify', verify_otp,name="verify_otp"),
 path('mail/re-send', resend_otp,name="resend_otp"),

 path("password-reset/", auth_views.PasswordResetView.as_view(
  template_name="password_reset.html", email_template_name="password_reset_email.html",html_email_template_name="password_reset_email.html"), name="password_reset"),
 path("password-reset/done/", auth_views.PasswordResetDoneView.as_view(
  template_name="password_reset_done.html"), name="password_reset_done"),
 path("password-reset-confirm/<uidb64>/<token>/", auth_views.PasswordResetConfirmView.as_view(
  template_name="password_reset_confirm.html"), name="password_reset_confirm"),
 path("password-reset-complete/", auth_views.PasswordResetCompleteView.as_view(
  template_name="password_reset_complete.html"), name="password_reset_complete"),

 path("change-password", CustomPasswordChangeView.as_view(), name="change_password"),
 path("change-password/done", auth_views.PasswordChangeDoneView.as_view(template_name="change_password_done.html"),
      name="password_change_done"),
 path('profile',profile, name='profile'),
]



import os
import random
import re

from django.core.files.storage import FileSystemStorage
from django.core.mail import EmailMultiAlternatives
from django.template.loader import render_to_string
from django.contrib.auth.views import PasswordChangeView
from django.core.mail import send_mail
from django.http import JsonResponse
from django.shortcuts import render, redirect
from django.contrib import messages
from django.contrib.auth import get_user_model, authenticate, login, logout
from django.contrib.auth.password_validation import validate_password
from django.core.exceptions import ValidationError
from django.db import transaction
from django.utils.html import strip_tags

from core.models import SystemUser, UserAddress
from ecommerce import settings
from products.models import Category

User = get_user_model()

def validate_password(password):
    """Custom password validation"""
    if len(password) < 8:
        raise ValidationError("Password must be at least 8 characters long.")
    if not re.search(r'[A-Z]', password):
        raise ValidationError("Password must contain at least one uppercase letter.")
    if not re.search(r'[a-z]', password):
        raise ValidationError("Password must contain at least one lowercase letter.")
    if not re.search(r'\d', password):
        raise ValidationError("Password must contain at least one digit.")
    if not re.search(r'[!@#$%^&*(),.?":{}|<>]', password):
        raise ValidationError("Password must contain at least one special character.")

def registration(request):
    if request.user.is_authenticated:
        return redirect("home")

    if request.method == "POST":
        # Basic info
        first_name = request.POST.get("first_name")
        last_name = request.POST.get("last_name")
        email = request.POST.get("email")
        phone_number = request.POST.get("phone_number")
        password1 = request.POST.get("password1")
        password2 = request.POST.get("password2")
        brand_name = request.POST.get("brand_name")
        licence = request.FILES.get("licence")

        # Address info
        street_address = request.POST.get("street_address")
        city = request.POST.get("city")
        state = request.POST.get("state")
        postal_code = request.POST.get("postal_code")
        country = request.POST.get("country")

        if SystemUser.objects.filter(email=email).exists():
            messages.error(request, "Email is already registered. Please use a different one.")
            return redirect("register")

        if password1 != password2:
            messages.error(request, "Passwords do not match.")
            return redirect("register")

        try:
            validate_password(password1)
        except ValidationError as e:
            messages.error(request, e.messages[0])
            return redirect("register")

        otp = str(random.randint(100000, 999999))
        request.session["otp"] = otp
        request.session["otp_required"] = True
        request.session["user_data"] = {
            "first_name": first_name,
            "last_name": last_name,
            "email": email,
            "phone_number": phone_number,
            "password": password1,
            "brand_name": brand_name,
            "licence_name": licence.name if licence else None,
        }
        request.session["user_address"] = {
            "street_address": street_address,
            "city": city,
            "state": state,
            "postal_code": postal_code,
            "country": country,
        }

        # Save licence temporarily
        if licence:
            fs = FileSystemStorage(location=os.path.join(settings.MEDIA_ROOT, 'temp_licences'))
            filename = fs.save(licence.name, licence)
            request.session["licence_path"] = fs.path(filename)

        # Send OTP to user
        html_message = render_to_string("send-mail.html", {"otp": otp, "first_name": first_name, "last_name": last_name})
        plain_message = strip_tags(html_message)

        email_message = EmailMultiAlternatives(
            subject="Your OTP Code",
            body=plain_message,
            from_email="noreply@example.com",
            to=[email],
        )
        email_message.attach_alternative(html_message, "text/html")
        email_message.send()

        # Send admin notification to Mrwireless2018@gmail.com
        admin_body = (
            f"New user registration:\n\n"
            f"Name: {first_name} {last_name}\n"
            f"Email: {email}\n"
            f"Phone: {phone_number}\n"
            f"Brand: {brand_name}\n"
            f"Address: {street_address}, {city}, {state}, {postal_code}, {country}\n"
            f"Licence: {licence.name if licence else 'Not uploaded'}"
        )

        admin_message = EmailMultiAlternatives(
            subject="New User Registration Alert",
            body=admin_body,
            from_email="noreply@example.com",
            to=["Mrwireless2018@gmail.com"],
        )
        admin_message.send()

        messages.success(request, "An OTP has been sent to your email.")
        return redirect("verify_otp")

    categories = Category.objects.prefetch_related("subcategories__brands").all()
    category = Category.objects.prefetch_related("subcategories").all()
    return render(request, "registration.html", {'categories': categories, 'category': category})
    

def verify_otp(request):
    if not request.session.get("otp_required"):
        messages.error(request, "You are not authorized to access this page.")
        return redirect("register")

    stored_otp = request.session.get("otp")

    if request.method == "POST":
        entered_otp = request.POST.get("otp")

        print(f"Stored OTP: {stored_otp}, Entered OTP: {entered_otp}")

        if stored_otp and entered_otp and entered_otp.strip() == stored_otp.strip():
            user_data = request.session.get("user_data")
            user_address = request.session.get("user_address")

            if not user_data or not user_address:
                messages.error(request, "Session expired. Please register again.")
                return redirect("register")

            # Create user with is_active=False
            user = SystemUser.objects.create_user(
                email=user_data["email"],
                password=user_data["password"],
                first_name=user_data["first_name"],
                last_name=user_data["last_name"],
                phone_number=user_data["phone_number"],
                brand_name=user_data["brand_name"],
            )

            # Handle licence file
            if "licence_path" in request.session:
                from django.core.files import File
                licence_path = request.session["licence_path"]
                with open(licence_path, 'rb') as f:
                    user.licence.save(user_data["licence_name"], File(f))
                os.remove(licence_path)

            # Save address
            UserAddress.objects.create(
                user=user,
                street_address=user_address["street_address"],
                city=user_address["city"],
                state=user_address["state"],
                postal_code=user_address["postal_code"],
                country=user_address["country"],
                is_primary=True,
            )

            user.save()

            # Clean up session
            for key in ["otp", "otp_required", "user_data", "licence_path", "user_address"]:
                request.session.pop(key, None)

            messages.success(request, "Registration successful! Please wait for admin approval.")
            return redirect("login")
        else:
            messages.error(request, "Invalid OTP. Try again.")

    return render(request, "verify_otp.html")


def resend_otp(request):
    """Resend OTP"""
    if "user_data" not in request.session:
        messages.error(request, "Session expired. Please register again.")
        return redirect("register")

    # Generate new OTP
    new_otp = str(random.randint(100000, 999999))
    request.session["otp"] = new_otp

    # Get user details from session
    user_data = request.session["user_data"]
    email = user_data["email"]
    first_name = user_data["first_name"]
    last_name = user_data["last_name"]

    # Render the email template
    html_message = render_to_string("send-mail.html",
                                    {"otp": new_otp, "first_name": first_name, "last_name": last_name})
    plain_message = strip_tags(html_message)

    email_message = EmailMultiAlternatives(
        subject="Your New OTP Code",
        body=plain_message,
        from_email="noreply@example.com",
        to=[email],
    )
    email_message.attach_alternative(html_message, "text/html")
    email_message.send()

    messages.success(request, "A new OTP has been sent to your email.")
    return redirect("verify_otp")


def user_login(request):
    if request.user.is_authenticated:
        return redirect("home")

    if request.method == "POST":
        email = request.POST.get("email")
        password = request.POST.get("password")

        user = authenticate(request, username=email, password=password)

        if user is not None:
            if user.is_active:
                login(request, user)
                messages.success(request, "Login successful!")
                return redirect("home")  # Change "home" to your actual homepage URL name
            else:
                messages.error(request, "Admin has not approved your account yet.")
        else:
            messages.error(request, "Invalid email or password. Please try again.")

    categories = Category.objects.prefetch_related("subcategories__brands").all()
    category = Category.objects.prefetch_related("subcategories").all()
    return render(request, "login.html", {'categories': categories, 'category': category})


def user_logout(request):
    logout(request)
    return redirect("login")

class CustomPasswordChangeView(PasswordChangeView):
    def form_valid(self, form):
        response = super().form_valid(form)
        return JsonResponse({"success": True, "message": "Password changed successfully!"})

    def form_invalid(self, form):
        return JsonResponse({"success": False, "errors": form.errors}, status=400)

def profile(request):
    if request.method == "POST":
        # Update SystemUser fields
        user = request.user
        user.first_name = request.POST.get('first_name', user.first_name)
        user.last_name = request.POST.get('last_name', user.last_name)
        user.phone_number = request.POST.get('phone_number', user.phone_number)
        user.brand_name = request.POST.get('brand_name', user.brand_name)

        # Handle licence file upload
        if 'licence' in request.FILES:
            licence_file = request.FILES['licence']
            fs = FileSystemStorage(location='media/licences/')
            filename = fs.save(licence_file.name, licence_file)
            user.licence = f'licences/{filename}'

        user.save()

        # Update UserProfile fields
        profile = user.profile
        profile.address = request.POST.get('address', profile.address)
        profile.date_of_birth = request.POST.get('date_of_birth', profile.date_of_birth) or None
        profile.gender = request.POST.get('gender', profile.gender) or None

        # Handle profile picture upload
        if 'profile_picture' in request.FILES:
            profile_picture = request.FILES['profile_picture']
            fs = FileSystemStorage(location='media/profile_pics/')
            filename = fs.save(profile_picture.name, profile_picture)
            profile.profile_picture = f'profile_pics/{filename}'

        profile.save()

        # Pass a flag to indicate the form was submitted
        return render(request, 'user-profile.html', {'form_submitted': True})

    return render(request, 'user-profile.html', {'form_submitted': False})




from django.contrib import admin
from django.contrib.auth.admin import UserAdmin as BaseUserAdmin
from .models import SystemUser, UserProfile, UserAddress
from django.utils.translation import gettext_lazy as _
import os

class UserProfileInline(admin.StackedInline):
    model = UserProfile
    can_delete = False
    verbose_name_plural = '🧍 User Profile Section'
    fk_name = 'user'
    fieldsets = (
        (None, {
            'fields': ('address', 'profile_picture', 'date_of_birth', 'gender'),
            'description': "This section contains the user's personal profile data.",
        }),
    )

class UserAddressInline(admin.StackedInline):
    model = UserAddress
    extra = 0
    verbose_name_plural = 'User Address Section'
    fk_name = 'user'
    fieldsets = (
        (None, {
            'fields': ('street_address', 'city', 'state', 'postal_code', 'country', 'is_primary'),
            'description': "Addresses associated with the user.",
        }),
    )

@admin.register(SystemUser)
class UserAdmin(BaseUserAdmin):
    inlines = [UserProfileInline, UserAddressInline]
    list_display = ('email', 'first_name', 'last_name', 'is_active', 'is_staff')
    list_filter = ('is_active', 'is_staff', 'profile__gender')
    fieldsets = (
        (None, {'fields': ('email', 'password')}),
        (_('Personal info'), {
            'fields': ('first_name', 'last_name', 'phone_number', 'brand_name', 'licence')
        }),
        (_('Permissions'), {
            'fields': ('is_active', 'is_staff', 'is_superuser', 'groups', 'user_permissions')
        }),
        (_('Important dates'), {'fields': ('last_login',)}),
    )
    add_fieldsets = (
        (None, {
            'classes': ('wide',),
            'fields': ('email', 'password1', 'password2'),
        }),
    )
    search_fields = ('email', 'first_name', 'last_name')
    ordering = ('email',)
    filter_horizontal = ('groups', 'user_permissions',)

@admin.register(UserProfile)
class UserProfileAdmin(admin.ModelAdmin):
    list_display = ('user', 'date_of_birth', 'gender')
    search_fields = ('user__email',)

@admin.register(UserAddress)
class UserAddressAdmin(admin.ModelAdmin):
    list_display = ('user', 'street_address', 'city', 'state', 'postal_code', 'country', 'is_primary')
    search_fields = ('user__email', 'city', 'state', 'country')
    list_filter = ('is_primary', 'country')



# Generated by Django 4.2 on 2025-03-24 18:54

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('core', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='systemuser',
            name='brand_name',
            field=models.CharField(blank=True, max_length=100, null=True),
        ),
        migrations.AddField(
            model_name='systemuser',
            name='licence',
            field=models.FileField(blank=True, null=True, upload_to='licences/'),
        ),
        migrations.AlterField(
            model_name='systemuser',
            name='is_active',
            field=models.BooleanField(default=False),
        ),
    ]



# Generated by Django 4.2 on 2025-05-10 06:11

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('core', '0002_systemuser_brand_name_systemuser_licence_and_more'),
    ]

    operations = [
        migrations.CreateModel(
            name='UserAddress',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('street_address', models.TextField()),
                ('city', models.CharField(max_length=100)),
                ('state', models.CharField(max_length=100)),
                ('postal_code', models.CharField(max_length=20)),
                ('country', models.CharField(max_length=100)),
                ('is_primary', models.BooleanField(default=False)),
                ('user', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='core_addresses', to=settings.AUTH_USER_MODEL)),
            ],
        ),
    ]



# Generated by Django 4.2 on 2025-02-15 13:38

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion
import uuid


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('auth', '0012_alter_user_first_name_max_length'),
    ]

    operations = [
        migrations.CreateModel(
            name='SystemUser',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('password', models.CharField(max_length=128, verbose_name='password')),
                ('last_login', models.DateTimeField(blank=True, null=True, verbose_name='last login')),
                ('is_superuser', models.BooleanField(default=False, help_text='Designates that this user has all permissions without explicitly assigning them.', verbose_name='superuser status')),
                ('uid', models.UUIDField(default=uuid.uuid4, editable=False, unique=True)),
                ('email', models.EmailField(max_length=254, unique=True)),
                ('first_name', models.CharField(blank=True, max_length=50, null=True)),
                ('last_name', models.CharField(blank=True, max_length=50, null=True)),
                ('phone_number', models.CharField(blank=True, max_length=15, null=True)),
                ('is_active', models.BooleanField(default=True)),
                ('is_staff', models.BooleanField(default=False)),
                ('groups', models.ManyToManyField(blank=True, help_text='The groups this user belongs to. A user will get all permissions granted to each of their groups.', related_name='user_set', related_query_name='user', to='auth.group', verbose_name='groups')),
                ('user_permissions', models.ManyToManyField(blank=True, help_text='Specific permissions for this user.', related_name='user_set', related_query_name='user', to='auth.permission', verbose_name='user permissions')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='UserProfile',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('address', models.TextField(blank=True, null=True)),
                ('profile_picture', models.ImageField(blank=True, null=True, upload_to='profile_pics/')),
                ('date_of_birth', models.DateField(blank=True, null=True)),
                ('gender', models.CharField(blank=True, choices=[('Male', 'Male'), ('Female', 'Female'), ('Other', 'Other')], max_length=10, null=True)),
                ('user', models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, related_name='profile', to=settings.AUTH_USER_MODEL)),
            ],
        ),
    ]



import uuid
from django.contrib.auth.models import AbstractBaseUser, BaseUserManager, PermissionsMixin
from django.db import models
from django.db.models.signals import post_save
from django.dispatch import receiver

class CustomUserManager(BaseUserManager):
    def create_user(self, email, password=None, **extra_fields):
        """Create and return a regular user with an email and password."""
        if not email:
            raise ValueError("The Email field must be set")
        email = self.normalize_email(email)
        user = self.model(email=email, **extra_fields)
        user.set_password(password)
        user.save(using=self._db)
        return user

    def create_superuser(self, email, password=None, **extra_fields):
        """Create and return a superuser with admin permissions."""
        extra_fields.setdefault("is_staff", True)
        extra_fields.setdefault("is_superuser", True)

        return self.create_user(email, password, **extra_fields)

class SystemUser(AbstractBaseUser, PermissionsMixin):
    uid = models.UUIDField(default=uuid.uuid4, unique=True, editable=False)
    email = models.EmailField(unique=True)
    first_name = models.CharField(max_length=50, blank=True, null=True)
    last_name = models.CharField(max_length=50, blank=True, null=True)
    phone_number = models.CharField(max_length=15, blank=True, null=True)
    brand_name = models.CharField(max_length=100, blank=True, null=True)
    licence = models.FileField(upload_to='licences/', blank=True, null=True)
    is_active = models.BooleanField(default=False)
    is_staff = models.BooleanField(default=False)

    objects = CustomUserManager()

    USERNAME_FIELD = "email"
    REQUIRED_FIELDS = []

    def __str__(self):
        return self.email

class UserAddress(models.Model):
    user = models.ForeignKey(SystemUser, on_delete=models.CASCADE, related_name='core_addresses')
    street_address = models.TextField(blank=True, null=True)
    city = models.CharField(max_length=100, blank=True, null=True)
    state = models.CharField(max_length=100, blank=True, null=True)
    postal_code = models.CharField(max_length=20, blank=True, null=True)
    country = models.CharField(max_length=100, blank=True, null=True)
    is_primary = models.BooleanField(default=False)

    def __str__(self):
        return f"{self.street_address}, {self.city}, {self.state}"


class UserProfile(models.Model):
    user = models.OneToOneField(SystemUser, on_delete=models.CASCADE, related_name="profile")
    address = models.TextField(blank=True, null=True)
    profile_picture = models.ImageField(upload_to="profile_pics/", blank=True, null=True)
    date_of_birth = models.DateField(blank=True, null=True)
    gender = models.CharField(max_length=10, choices=[("Male", "Male"), ("Female", "Female"), ("Other", "Other")], blank=True, null=True)

    def __str__(self):
        return f"Profile of {self.user.email}"

@receiver(post_save, sender=SystemUser)
def create_user_profile(sender, instance, created, **kwargs):
    if created:
        UserProfile.objects.create(user=instance)

@receiver(post_save, sender=SystemUser)
def save_user_profile(sender, instance, **kwargs):
    """Saves the UserProfile when CustomUser is saved"""
    instance.profile.save()


# Generated by Django 4.2 on 2025-05-12 08:41

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('core', '0003_useraddress'),
    ]

    operations = [
        migrations.AlterField(
            model_name='useraddress',
            name='city',
            field=models.CharField(blank=True, max_length=100, null=True),
        ),
        migrations.AlterField(
            model_name='useraddress',
            name='country',
            field=models.CharField(blank=True, max_length=100, null=True),
        ),
        migrations.AlterField(
            model_name='useraddress',
            name='postal_code',
            field=models.CharField(blank=True, max_length=20, null=True),
        ),
        migrations.AlterField(
            model_name='useraddress',
            name='state',
            field=models.CharField(blank=True, max_length=100, null=True),
        ),
        migrations.AlterField(
            model_name='useraddress',
            name='street_address',
            field=models.TextField(blank=True, null=True),
        ),
    ]
